@@ -74,11 +74,15 @@ int spa_alsa_close(struct state *state)
 		return 0;
 
 	spa_log_debug(state->log, NAME" %p: Device '%s' closing", state, state->props.device);
-	CHECK(snd_pcm_close(state->hndl), "%s: close failed", state->props.device);
+	if ((err = snd_pcm_close(state->hndl)) < 0)
+		spa_log_warn(state->log, "%s: close failed: %s", state->props.device,
+				snd_strerror(err));
 
-	CHECK(snd_output_close(state->output), "output close failed");
+	if ((err = snd_output_close(state->output)) < 0)
+		spa_log_warn(state->log, "output close failed: %s", snd_strerror(err));
 
 	spa_system_close(state->data_system, state->timerfd);
+
 	state->opened = false;
 
 	return err;
@@ -620,7 +624,7 @@ static int alsa_recover(struct state *state, int err)
 	if (SPA_UNLIKELY((res = snd_pcm_status(state->hndl, status)) < 0)) {
 		spa_log_error(state->log, NAME" %p: snd_pcm_status error: %s",
 				state, snd_strerror(res));
-		return res;
+		goto recover;
 	}
 
 	st = snd_pcm_status_get_state(status);
@@ -646,12 +650,17 @@ static int alsa_recover(struct state *state, int err)
 		state->sample_count += missing ? missing : state->threshold;
 		break;
 	}
+	case SND_PCM_STATE_SUSPENDED:
+		spa_log_info(state->log, NAME" %p: recover from state %s",
+				state, snd_pcm_state_name(st));
+		break;
 	default:
-		spa_log_error(state->log, NAME" %p: recover from error state %d",
-				state, st);
+		spa_log_error(state->log, NAME" %p: recover from error state %s",
+				state, snd_pcm_state_name(st));
 		break;
 	}
 
+recover:
 	if (SPA_UNLIKELY((res = snd_pcm_recover(state->hndl, err, true)) < 0)) {
 		spa_log_error(state->log, NAME" %p: snd_pcm_recover error: %s",
 				state, snd_strerror(res));
@@ -1160,6 +1169,7 @@ static int handle_capture(struct state *state, uint64_t nsec,
 
 		io->buffer_id = b->id;
 		io->status = SPA_STATUS_HAVE_DATA;
+		spa_log_trace_fp(state->log, NAME" %p: output buffer:%d", state, b->id);
 	}
 	spa_node_call_ready(&state->callbacks, SPA_STATUS_HAVE_DATA);
 	return 0;
