@@ -84,205 +84,36 @@ static inline void add_dict(struct spa_pod_builder *builder, const char *key, co
 	spa_pod_builder_string(builder, val);
 }
 
-static uint8_t a2dp_default_bitpool(struct spa_bt_monitor *monitor, uint8_t freq, uint8_t mode) {
-	/* These bitpool values were chosen based on the A2DP spec recommendation */
-	switch (freq) {
-	case SBC_SAMPLING_FREQ_16000:
-        case SBC_SAMPLING_FREQ_32000:
-		return 53;
-
-	case SBC_SAMPLING_FREQ_44100:
-		switch (mode) {
-		case SBC_CHANNEL_MODE_MONO:
-		case SBC_CHANNEL_MODE_DUAL_CHANNEL:
-			return 31;
-
-		case SBC_CHANNEL_MODE_STEREO:
-		case SBC_CHANNEL_MODE_JOINT_STEREO:
-			return 53;
-		}
-
-		spa_log_warn(monitor->log, "Invalid channel mode %u", mode);
-		return 53;
-	case SBC_SAMPLING_FREQ_48000:
-		switch (mode) {
-		case SBC_CHANNEL_MODE_MONO:
-		case SBC_CHANNEL_MODE_DUAL_CHANNEL:
-			return 29;
-
-		case SBC_CHANNEL_MODE_STEREO:
-		case SBC_CHANNEL_MODE_JOINT_STEREO:
-			return 51;
-		}
-
-		spa_log_warn(monitor->log, "Invalid channel mode %u", mode);
-		return 51;
-	}
-	spa_log_warn(monitor->log, "Invalid sampling freq %u", freq);
-	return 53;
-}
-
-static int select_configuration_sbc(struct spa_bt_monitor *monitor, void *capabilities, size_t size, void *config)
+static const struct a2dp_codec *a2dp_endpoint_to_codec(const char *endpoint)
 {
-	a2dp_sbc_t *cap, conf;
-	int bitpool;
+	const char *codec_name;
+	int i;
 
-	if (size < sizeof(conf)) {
-		spa_log_error(monitor->log, "Capabilities array has invalid size");
-		return -ENOSPC;
+	if (strstr(endpoint, A2DP_SINK_ENDPOINT "/") == endpoint)
+		codec_name = endpoint + strlen(A2DP_SINK_ENDPOINT "/");
+	else if (strstr(endpoint, A2DP_SOURCE_ENDPOINT "/") == endpoint)
+		codec_name = endpoint + strlen(A2DP_SOURCE_ENDPOINT "/");
+	else
+		return NULL;
+
+	for (i = 0; a2dp_codecs[i]; i++) {
+		const struct a2dp_codec *codec = a2dp_codecs[i];
+		if (strcmp(codec->name, codec_name) == 0)
+			return codec;
 	}
-	cap = capabilities;
-	conf = *cap;
-
-	if (conf.frequency & SBC_SAMPLING_FREQ_48000)
-		conf.frequency = SBC_SAMPLING_FREQ_48000;
-	else if (conf.frequency & SBC_SAMPLING_FREQ_44100)
-		conf.frequency = SBC_SAMPLING_FREQ_44100;
-	else if (conf.frequency & SBC_SAMPLING_FREQ_32000)
-		conf.frequency = SBC_SAMPLING_FREQ_32000;
-	else if (conf.frequency & SBC_SAMPLING_FREQ_16000)
-		conf.frequency = SBC_SAMPLING_FREQ_16000;
-	else {
-		spa_log_error(monitor->log, "No supported sampling frequencies: 0x%x", conf.frequency);
-		return -ENOTSUP;
-	}
-
-	if (conf.channel_mode & SBC_CHANNEL_MODE_JOINT_STEREO)
-		conf.channel_mode = SBC_CHANNEL_MODE_JOINT_STEREO;
-	else if (conf.channel_mode & SBC_CHANNEL_MODE_STEREO)
-		conf.channel_mode = SBC_CHANNEL_MODE_STEREO;
-	else if (conf.channel_mode & SBC_CHANNEL_MODE_DUAL_CHANNEL)
-		conf.channel_mode = SBC_CHANNEL_MODE_DUAL_CHANNEL;
-	else if (conf.channel_mode & SBC_CHANNEL_MODE_MONO)
-		conf.channel_mode = SBC_CHANNEL_MODE_MONO;
-	else {
-		spa_log_error(monitor->log, "No supported channel modes: 0x%x", conf.channel_mode);
-		return -ENOTSUP;
-	}
-
-	if (conf.block_length & SBC_BLOCK_LENGTH_16)
-		conf.block_length = SBC_BLOCK_LENGTH_16;
-	else if (conf.block_length & SBC_BLOCK_LENGTH_12)
-		conf.block_length = SBC_BLOCK_LENGTH_12;
-	else if (conf.block_length & SBC_BLOCK_LENGTH_8)
-		conf.block_length = SBC_BLOCK_LENGTH_8;
-	else if (conf.block_length & SBC_BLOCK_LENGTH_4)
-		conf.block_length = SBC_BLOCK_LENGTH_4;
-	else {
-		spa_log_error(monitor->log, "No supported block lengths: 0x%x", conf.block_length);
-		return -ENOTSUP;
-	}
-
-	if (conf.subbands & SBC_SUBBANDS_8)
-		conf.subbands = SBC_SUBBANDS_8;
-	else if (conf.subbands & SBC_SUBBANDS_4)
-		conf.subbands = SBC_SUBBANDS_4;
-	else {
-		spa_log_error(monitor->log, "No supported subbands: 0x%x", conf.subbands);
-		return -ENOTSUP;
-	}
-
-	if (conf.allocation_method & SBC_ALLOCATION_LOUDNESS)
-		conf.allocation_method = SBC_ALLOCATION_LOUDNESS;
-	else if (conf.allocation_method & SBC_ALLOCATION_SNR)
-		conf.allocation_method = SBC_ALLOCATION_SNR;
-	else {
-		spa_log_error(monitor->log, "No supported allocation: 0x%x", conf.allocation_method);
-		return -ENOTSUP;
-	}
-
-	bitpool = a2dp_default_bitpool(monitor, conf.frequency, conf.channel_mode);
-
-	conf.min_bitpool = SPA_MAX(MIN_BITPOOL, conf.min_bitpool);
-	conf.max_bitpool = SPA_MIN(bitpool, conf.max_bitpool);
-	memcpy(config, &conf, size);
-
-	spa_log_debug(monitor->log, "SelectConfiguration(): %d %d %d %d ",
-			conf.frequency, conf.channel_mode, conf.min_bitpool, conf.max_bitpool);
-
-	return 0;
-}
-
-static int select_configuration_aac(struct spa_bt_monitor *monitor, void *capabilities, size_t size, void *config)
-{
-	a2dp_aac_t *cap, conf;
-	int freq;
-
-	if (size < sizeof(conf)) {
-		spa_log_error(monitor->log, "Capabilities array has invalid size");
-		return -ENOSPC;
-	}
-	cap = capabilities;
-	conf = *cap;
-
-	if (conf.object_type & AAC_OBJECT_TYPE_MPEG2_AAC_LC)
-		conf.object_type = AAC_OBJECT_TYPE_MPEG2_AAC_LC;
-	else if (conf.object_type & AAC_OBJECT_TYPE_MPEG4_AAC_LC)
-		conf.object_type = AAC_OBJECT_TYPE_MPEG4_AAC_LC;
-	else if (conf.object_type & AAC_OBJECT_TYPE_MPEG4_AAC_LTP)
-		conf.object_type = AAC_OBJECT_TYPE_MPEG4_AAC_LTP;
-	else if (conf.object_type & AAC_OBJECT_TYPE_MPEG4_AAC_SCA)
-		conf.object_type = AAC_OBJECT_TYPE_MPEG4_AAC_SCA;
-	else {
-		spa_log_error(monitor->log, "No supported object type: 0x%x", conf.object_type);
-		return -ENOTSUP;
-	}
-
-	freq = AAC_GET_FREQUENCY(conf);
-	if (freq & AAC_SAMPLING_FREQ_48000)
-		freq = AAC_SAMPLING_FREQ_48000;
-	else if (freq & AAC_SAMPLING_FREQ_44100)
-		freq = AAC_SAMPLING_FREQ_44100;
-	else if (freq & AAC_SAMPLING_FREQ_64000)
-		freq = AAC_SAMPLING_FREQ_64000;
-	else if (freq & AAC_SAMPLING_FREQ_32000)
-		freq = AAC_SAMPLING_FREQ_32000;
-	else if (freq & AAC_SAMPLING_FREQ_88200)
-		freq = AAC_SAMPLING_FREQ_88200;
-	else if (freq & AAC_SAMPLING_FREQ_96000)
-		freq = AAC_SAMPLING_FREQ_96000;
-	else if (freq & AAC_SAMPLING_FREQ_24000)
-		freq = AAC_SAMPLING_FREQ_24000;
-	else if (freq & AAC_SAMPLING_FREQ_22050)
-		freq = AAC_SAMPLING_FREQ_22050;
-	else if (freq & AAC_SAMPLING_FREQ_16000)
-		freq = AAC_SAMPLING_FREQ_16000;
-	else if (freq & AAC_SAMPLING_FREQ_12000)
-		freq = AAC_SAMPLING_FREQ_12000;
-	else if (freq & AAC_SAMPLING_FREQ_11025)
-		freq = AAC_SAMPLING_FREQ_11025;
-	else if (freq & AAC_SAMPLING_FREQ_8000)
-		freq = AAC_SAMPLING_FREQ_8000;
-	else {
-		spa_log_error(monitor->log, "No supported sampling frequency: 0x%0x", freq);
-		return -ENOTSUP;
-	}
-	AAC_SET_FREQUENCY(conf, freq);
-
-	if (conf.channels & AAC_CHANNELS_2)
-		conf.channels = AAC_CHANNELS_2;
-	else if (conf.channels & AAC_CHANNELS_1)
-		conf.channels = AAC_CHANNELS_1;
-	else {
-		spa_log_error(monitor->log, "No supported channels: 0x%0x", conf.channels);
-		return -ENOTSUP;
-	}
-	memcpy(config, &conf, size);
-
-	spa_log_debug(monitor->log, "SelectConfiguration() %d %d %d", conf.object_type, freq, conf.channels);
-
-	return 0;
+	return NULL;
 }
 
 static DBusHandlerResult endpoint_select_configuration(DBusConnection *conn, DBusMessage *m, void *userdata)
 {
 	struct spa_bt_monitor *monitor = userdata;
 	const char *path;
-	uint8_t *cap, config[16];
+	uint8_t *cap, config[A2DP_MAX_CAPS_SIZE];
 	uint8_t *pconf = (uint8_t *) config;
 	DBusMessage *r;
 	DBusError err;
 	int size, res;
+	const struct a2dp_codec *codec;
 
 	dbus_error_init(&err);
 
@@ -294,12 +125,12 @@ static DBusHandlerResult endpoint_select_configuration(DBusConnection *conn, DBu
 		dbus_error_free(&err);
 		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 	}
+	spa_log_info(monitor->log, "%p: select conf %d", monitor, size);
 
-	if (strstr(path, "/A2DP/SBC/") == path) {
-		res = select_configuration_sbc(monitor, cap, size, config);
-	} else if (strstr(path, "/A2DP/MPEG24/") == path) {
-		res = select_configuration_aac(monitor, cap, size, config);
-	} else
+	codec = a2dp_endpoint_to_codec(path);
+	if (codec != NULL)
+		res = codec->select_config(0, cap, size, NULL, config);
+	else
 		res = -ENOTSUP;
 
 	if (res < 0) {
@@ -308,6 +139,7 @@ static DBusHandlerResult endpoint_select_configuration(DBusConnection *conn, DBu
 			return DBUS_HANDLER_RESULT_NEED_MEMORY;
 		goto exit_send;
 	}
+	size = res;
 
 	if ((r = dbus_message_new_method_return(m)) == NULL)
 		return DBUS_HANDLER_RESULT_NEED_MEMORY;
@@ -333,6 +165,16 @@ static struct spa_bt_adapter *adapter_find(struct spa_bt_monitor *monitor, const
 	return NULL;
 }
 
+static bool check_iter_signature(DBusMessageIter *it, const char *sig)
+{
+	char *v;
+	int res;
+	v = dbus_message_iter_get_signature(it);
+	res = strcmp(v, sig);
+	dbus_free(v);
+	return res == 0;
+}
+
 static int adapter_update_props(struct spa_bt_adapter *adapter,
 				DBusMessageIter *props_iter,
 				DBusMessageIter *invalidated_iter)
@@ -396,7 +238,7 @@ static int adapter_update_props(struct spa_bt_adapter *adapter,
 		else if (strcmp(key, "UUIDs") == 0) {
 			DBusMessageIter iter;
 
-			if (strcmp(dbus_message_iter_get_signature(&it[1]), "as") != 0)
+			if (!check_iter_signature(&it[1], "as"))
 				goto next;
 
 			dbus_message_iter_recurse(&it[1], &iter);
@@ -780,7 +622,7 @@ static int device_update_props(struct spa_bt_device *device,
 		else if (strcmp(key, "UUIDs") == 0) {
 			DBusMessageIter iter;
 
-			if (strcmp(dbus_message_iter_get_signature(&it[1]), "as") != 0)
+			if (!check_iter_signature(&it[1], "as"))
 				goto next;
 
 			dbus_message_iter_recurse(&it[1], &iter);
@@ -945,7 +787,7 @@ static int transport_update_props(struct spa_bt_transport *transport,
 			char *value;
 			int len;
 
-			if (strcmp(dbus_message_iter_get_signature(&it[1]), "ay") != 0)
+			if (!check_iter_signature(&it[1], "ay"))
 				goto next;
 
 			dbus_message_iter_recurse(&it[1], &iter);
@@ -1085,16 +927,24 @@ static DBusHandlerResult endpoint_set_configuration(DBusConnection *conn,
 		const char *path, DBusMessage *m, void *userdata)
 {
 	struct spa_bt_monitor *monitor = userdata;
-	const char *transport_path;
+	const char *transport_path, *endpoint;
 	DBusMessageIter it[2];
 	DBusMessage *r;
 	struct spa_bt_transport *transport;
 	bool is_new = false;
+	const struct a2dp_codec *codec;
 
 	if (!dbus_message_has_signature(m, "oa{sv}")) {
 		spa_log_warn(monitor->log, "invalid SetConfiguration() signature");
 		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 	}
+	endpoint = dbus_message_get_path(m);
+
+	codec = a2dp_endpoint_to_codec(endpoint);
+	if (codec == NULL) {
+		spa_log_warn(monitor->log, "unknown SetConfiguration() codec");
+		return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	}
 
 	dbus_message_iter_init(m, &it[0]);
 	dbus_message_iter_get_basic(&it[0], &transport_path);
@@ -1112,6 +962,7 @@ static DBusHandlerResult endpoint_set_configuration(DBusConnection *conn,
 		spa_bt_transport_set_implementation(transport, &transport_impl, transport);
 	}
 	transport_update_props(transport, &it[1], NULL);
+	transport->a2dp_codec = codec;
 
 	if (transport->device == NULL) {
 		spa_log_warn(monitor->log, "no device found for transport");
@@ -1230,7 +1081,7 @@ static DBusHandlerResult endpoint_handler(DBusConnection *c, DBusMessage *m, voi
 	return res;
 }
 
-static void register_endpoint_reply(DBusPendingCall *pending, void *user_data)
+static void bluez_register_endpoint_reply(DBusPendingCall *pending, void *user_data)
 {
 	struct spa_bt_monitor *monitor = user_data;
 	DBusMessage *r;
@@ -1249,117 +1100,95 @@ static void register_endpoint_reply(DBusPendingCall *pending, void *user_data)
 		goto finish;
 	}
 
-      finish:
+	finish:
 	dbus_message_unref(r);
-        dbus_pending_call_unref(pending);
+	dbus_pending_call_unref(pending);
 }
 
-static int register_a2dp_endpoint(struct spa_bt_monitor *monitor,
-				  const char *path,
-				  const char *uuid,
-				  enum spa_bt_profile profile,
-				  uint16_t codec,
-				  const void *configuration,
-				  size_t configuration_size)
-{
-	const char *profile_path;
-	char *object_path, *str;
-	const DBusObjectPathVTable vtable_endpoint = {
-		.message_function = endpoint_handler,
-	};
+static void append_basic_variant_dict_entry(DBusMessageIter *dict, int key_type_int, void* key, int variant_type_int, const char* variant_type_str, void* variant) {
+	DBusMessageIter dict_entry_it, variant_it;
+	dbus_message_iter_open_container(dict, DBUS_TYPE_DICT_ENTRY, NULL, &dict_entry_it);
+	dbus_message_iter_append_basic(&dict_entry_it, key_type_int, key);
+
+	dbus_message_iter_open_container(&dict_entry_it, DBUS_TYPE_VARIANT, variant_type_str, &variant_it);
+	dbus_message_iter_append_basic(&variant_it, variant_type_int, variant);
+	dbus_message_iter_close_container(&dict_entry_it, &variant_it);
+	dbus_message_iter_close_container(dict, &dict_entry_it);
+}
+
+static void append_basic_array_variant_dict_entry(DBusMessageIter *dict, int key_type_int, void* key, const char* variant_type_str, const char* array_type_str, int array_type_int, void* data, int data_size) {
+	DBusMessageIter dict_entry_it, variant_it, array_it;
+	dbus_message_iter_open_container(dict, DBUS_TYPE_DICT_ENTRY, NULL, &dict_entry_it);
+	dbus_message_iter_append_basic(&dict_entry_it, key_type_int, key);
+
+	dbus_message_iter_open_container(&dict_entry_it, DBUS_TYPE_VARIANT, variant_type_str, &variant_it);
+	dbus_message_iter_open_container(&variant_it, DBUS_TYPE_ARRAY, array_type_str, &array_it);
+	dbus_message_iter_append_fixed_array (&array_it, array_type_int, &data, data_size);
+	dbus_message_iter_close_container(&variant_it, &array_it);
+	dbus_message_iter_close_container(&dict_entry_it, &variant_it);
+	dbus_message_iter_close_container(dict, &dict_entry_it);
+}
+
+static int bluez_register_endpoint(struct spa_bt_monitor *monitor,
+                             const char *path, const char *uuid,
+                             const struct a2dp_codec *codec, const char *object_path) {
+	char* str;
 	DBusMessage *m;
-	DBusMessageIter it[5];
+	DBusMessageIter object_it, dict_it;
 	DBusPendingCall *call;
+	uint8_t caps[A2DP_MAX_CAPS_SIZE];
+	int caps_size;
+	uint16_t codec_id = codec->id.codec_id;
 
-	switch (profile) {
-	case SPA_BT_PROFILE_A2DP_SOURCE:
-		switch (codec) {
-		case A2DP_CODEC_SBC:
-			profile_path = "/A2DP/SBC/Source";
-			break;
-		case A2DP_CODEC_MPEG24:
-			/* We don't support MPEG24 for now */
-			return -ENOTSUP;
-			/* profile_path = "/A2DP/MPEG24/Source";
-			break; */
-		default:
-			return -ENOTSUP;
-		}
-		break;
-	case SPA_BT_PROFILE_A2DP_SINK:
-		switch (codec) {
-		case A2DP_CODEC_SBC:
-			profile_path = "/A2DP/SBC/Sink";
-			break;
-
-		case A2DP_CODEC_MPEG24:
-			/* We don't support MPEG24 for now */
-			return -ENOTSUP;
-			/* profile_path = "/A2DP/MPEG24/Sink";
-			break; */
-		default:
-			return -ENOTSUP;
-		}
-		break;
-	default:
-		return -ENOTSUP;
-	}
-
-	object_path = spa_aprintf("%s/%d", profile_path, monitor->count++);
-	if (object_path == NULL)
-		return -errno;
-
-	spa_log_debug(monitor->log, "Registering endpoint: %s", object_path);
-
-	if (!dbus_connection_register_object_path(monitor->conn,
-						  object_path,
-						  &vtable_endpoint, monitor))
-		return -EIO;
+	caps_size = codec->fill_caps(0, caps);
+	if (caps_size < 0)
+		return caps_size;
 
 	m = dbus_message_new_method_call(BLUEZ_SERVICE,
-					 path,
-					 BLUEZ_MEDIA_INTERFACE,
-					 "RegisterEndpoint");
+	                                 path,
+	                                 BLUEZ_MEDIA_INTERFACE,
+	                                 "RegisterEndpoint");
 	if (m == NULL)
 		return -EIO;
 
-	dbus_message_iter_init_append(m, &it[0]);
-	dbus_message_iter_append_basic(&it[0], DBUS_TYPE_OBJECT_PATH, &object_path);
+	dbus_message_iter_init_append(m, &object_it);
+	dbus_message_iter_append_basic(&object_it, DBUS_TYPE_OBJECT_PATH, &object_path);
 
-	dbus_message_iter_open_container(&it[0], DBUS_TYPE_ARRAY, "{sv}", &it[1]);
+	dbus_message_iter_open_container(&object_it, DBUS_TYPE_ARRAY, "{sv}", &dict_it);
 
-	dbus_message_iter_open_container(&it[1], DBUS_TYPE_DICT_ENTRY, NULL, &it[2]);
 	str = "UUID";
-	dbus_message_iter_append_basic(&it[2], DBUS_TYPE_STRING, &str);
-	dbus_message_iter_open_container(&it[2], DBUS_TYPE_VARIANT, "s", &it[3]);
-	dbus_message_iter_append_basic(&it[3], DBUS_TYPE_STRING, &uuid);
-	dbus_message_iter_close_container(&it[2], &it[3]);
-	dbus_message_iter_close_container(&it[1], &it[2]);
-
-	dbus_message_iter_open_container(&it[1], DBUS_TYPE_DICT_ENTRY, NULL, &it[2]);
+	append_basic_variant_dict_entry(&dict_it, DBUS_TYPE_STRING, &str, DBUS_TYPE_STRING, "s", &uuid);
 	str = "Codec";
-	dbus_message_iter_append_basic(&it[2], DBUS_TYPE_STRING, &str);
-	dbus_message_iter_open_container(&it[2], DBUS_TYPE_VARIANT, "y", &it[3]);
-	dbus_message_iter_append_basic(&it[3], DBUS_TYPE_BYTE, &codec);
-	dbus_message_iter_close_container(&it[2], &it[3]);
-	dbus_message_iter_close_container(&it[1], &it[2]);
-
-	dbus_message_iter_open_container(&it[1], DBUS_TYPE_DICT_ENTRY, NULL, &it[2]);
+	append_basic_variant_dict_entry(&dict_it, DBUS_TYPE_STRING, &str, DBUS_TYPE_BYTE, "y", &codec_id);
 	str = "Capabilities";
-	dbus_message_iter_append_basic(&it[2], DBUS_TYPE_STRING, &str);
-	dbus_message_iter_open_container(&it[2], DBUS_TYPE_VARIANT, "ay", &it[3]);
-	dbus_message_iter_open_container(&it[3], DBUS_TYPE_ARRAY, "y", &it[4]);
-	dbus_message_iter_append_fixed_array (&it[4], DBUS_TYPE_BYTE,
-			&configuration, configuration_size);
-	dbus_message_iter_close_container(&it[3], &it[4]);
-	dbus_message_iter_close_container(&it[2], &it[3]);
-	dbus_message_iter_close_container(&it[1], &it[2]);
-	dbus_message_iter_close_container(&it[0], &it[1]);
+	append_basic_array_variant_dict_entry(&dict_it, DBUS_TYPE_STRING, &str, "ay", "y", DBUS_TYPE_BYTE, caps, caps_size);
+
+	dbus_message_iter_close_container(&object_it, &dict_it);
 
 	dbus_connection_send_with_reply(monitor->conn, m, &call, -1);
-	dbus_pending_call_set_notify(call, register_endpoint_reply, monitor, NULL);
+	dbus_pending_call_set_notify(call, bluez_register_endpoint_reply, monitor, NULL);
 	dbus_message_unref(m);
-	free(object_path);
+
+	return 0;
+}
+
+static int register_a2dp_endpoint(struct spa_bt_monitor *monitor,
+		const struct a2dp_codec *codec, const char *endpoint, char** object_path)
+{
+	const DBusObjectPathVTable vtable_endpoint = {
+		.message_function = endpoint_handler,
+	};
+
+	*object_path = spa_aprintf("%s/%s", endpoint, codec->name);
+	if (*object_path == NULL)
+		return -errno;
+
+	spa_log_info(monitor->log, "Registering endpoint: %s", *object_path);
+
+	if (!dbus_connection_register_object_path(monitor->conn,
+	                                          *object_path,
+	                                          &vtable_endpoint, monitor))
+		return -EIO;
 
 	return 0;
 }
@@ -1367,34 +1196,248 @@ static int register_a2dp_endpoint(struct spa_bt_monitor *monitor,
 static int adapter_register_endpoints(struct spa_bt_adapter *a)
 {
 	struct spa_bt_monitor *monitor = a->monitor;
+	char *endpoint_path = NULL;
+	int i;
+	int err = 0;
 
-	/* We don't support MPEG24 for now */
-	/*
-	register_a2dp_endpoint(monitor, a->path,
-			       SPA_BT_UUID_A2DP_SOURCE,
-			       SPA_BT_PROFILE_A2DP_SOURCE,
-			       A2DP_CODEC_MPEG24,
-			       &bluez_a2dp_aac, sizeof(bluez_a2dp_aac));
-	register_a2dp_endpoint(monitor, a->path,
-			       SPA_BT_UUID_A2DP_SINK,
-			       SPA_BT_PROFILE_A2DP_SINK,
-			       A2DP_CODEC_MPEG24,
-			       &bluez_a2dp_aac, sizeof(bluez_a2dp_aac));
-	*/
+	if (a->endpoints_registered)
+	    return err;
 
-	register_a2dp_endpoint(monitor, a->path,
-			       SPA_BT_UUID_A2DP_SOURCE,
-			       SPA_BT_PROFILE_A2DP_SOURCE,
-			       A2DP_CODEC_SBC,
-			       &bluez_a2dp_sbc, sizeof(bluez_a2dp_sbc));
-	register_a2dp_endpoint(monitor, a->path,
-			       SPA_BT_UUID_A2DP_SINK,
-			       SPA_BT_PROFILE_A2DP_SINK,
-			       A2DP_CODEC_SBC,
-			       &bluez_a2dp_sbc, sizeof(bluez_a2dp_sbc));
-	return 0;
+	/* The legacy bluez5 api doesn't support codec switching
+	 * It doesn't make sense to register codecs other than SBC
+	 * as bluez5 will probably use SBC anyway and we have no control over it
+	 * let's incentivize users to upgrade their bluez5 daemon
+	 * if they want proper a2dp codec support
+	 * */
+	spa_log_warn(monitor->log, "Using legacy bluez5 API for A2DP - only SBC will be supported. "
+                               "Please upgrade bluez5.");
+
+	for (i = 0; a2dp_codecs[i]; i++) {
+		const struct a2dp_codec *codec = a2dp_codecs[i];
+
+		if (codec->id.codec_id != A2DP_CODEC_SBC)
+			continue;
+
+		if ((err = register_a2dp_endpoint(monitor, codec, A2DP_SOURCE_ENDPOINT, &endpoint_path)))
+			goto out;
+		if ((err = bluez_register_endpoint(monitor, a->path,
+		                                   SPA_BT_UUID_A2DP_SOURCE,
+		                                   codec,
+		                                   endpoint_path)))
+			goto out;
+		free(endpoint_path);
+		endpoint_path = NULL;
+
+		if ((err = register_a2dp_endpoint(monitor, codec, A2DP_SINK_ENDPOINT, &endpoint_path)))
+			goto out;
+		if ((err = bluez_register_endpoint(monitor, a->path,
+		                                   SPA_BT_UUID_A2DP_SINK,
+		                                   codec,
+		                                   endpoint_path)))
+			goto out;
+
+		a->endpoints_registered = true;
+		break;
+	}
+
+	if (!a->endpoints_registered) {
+		/* Should never happen as SBC support is always enabled */
+		spa_log_error(monitor->log, "Broken Pipewire build - unable to locate SBC codec");
+		err = -ENOSYS;
+	}
+
+	out:
+	if (err) {
+		spa_log_error(monitor->log, "Failed to register bluez5 endpoints");
+	}
+	if (endpoint_path)
+		free(endpoint_path);
+	return err;
 }
 
+static void append_a2dp_object(DBusMessageIter *iter, const char *endpoint,
+		const char *uuid, uint8_t codec_id, uint8_t *caps, size_t caps_size)
+{
+	char* str;
+	const char *interface_name = BLUEZ_MEDIA_ENDPOINT_INTERFACE;
+	DBusMessageIter object, array, entry, dict;
+
+	dbus_message_iter_open_container(iter, DBUS_TYPE_DICT_ENTRY, NULL, &object);
+	dbus_message_iter_append_basic(&object, DBUS_TYPE_OBJECT_PATH, &endpoint);
+
+	dbus_message_iter_open_container(&object, DBUS_TYPE_ARRAY, "{sa{sv}}", &array);
+
+	dbus_message_iter_open_container(&array, DBUS_TYPE_DICT_ENTRY, NULL, &entry);
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &interface_name);
+
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_ARRAY, "{sv}", &dict);
+
+	str = "UUID";
+	append_basic_variant_dict_entry(&dict, DBUS_TYPE_STRING, &str, DBUS_TYPE_STRING, "s", &uuid);
+	str = "Codec";
+	append_basic_variant_dict_entry(&dict, DBUS_TYPE_STRING, &str, DBUS_TYPE_BYTE, "y", &codec_id);
+	str = "Capabilities";
+	append_basic_array_variant_dict_entry(&dict, DBUS_TYPE_STRING, &str, "ay", "y", DBUS_TYPE_BYTE, caps, caps_size);
+
+	dbus_message_iter_close_container(&entry, &dict);
+	dbus_message_iter_close_container(&array, &entry);
+	dbus_message_iter_close_container(&object, &array);
+	dbus_message_iter_close_container(iter, &object);
+}
+
+static DBusHandlerResult object_manager_handler(DBusConnection *c, DBusMessage *m, void *user_data)
+{
+	struct spa_bt_monitor *monitor = user_data;
+	const char *path, *interface, *member;
+	char *endpoint;
+	DBusMessage *r;
+	DBusMessageIter iter, array;
+	DBusHandlerResult res;
+	int i;
+
+	path = dbus_message_get_path(m);
+	interface = dbus_message_get_interface(m);
+	member = dbus_message_get_member(m);
+
+	spa_log_debug(monitor->log, "dbus: path=%s, interface=%s, member=%s", path, interface, member);
+
+	if (dbus_message_is_method_call(m, "org.freedesktop.DBus.Introspectable", "Introspect")) {
+		const char *xml = OBJECT_MANAGER_INTROSPECT_XML;
+
+		if ((r = dbus_message_new_method_return(m)) == NULL)
+			return DBUS_HANDLER_RESULT_NEED_MEMORY;
+		if (!dbus_message_append_args(r, DBUS_TYPE_STRING, &xml, DBUS_TYPE_INVALID))
+			return DBUS_HANDLER_RESULT_NEED_MEMORY;
+		if (!dbus_connection_send(monitor->conn, r, NULL))
+			return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
+		dbus_message_unref(r);
+		res = DBUS_HANDLER_RESULT_HANDLED;
+	}
+	else if (dbus_message_is_method_call(m, "org.freedesktop.DBus.ObjectManager", "GetManagedObjects")) {
+		if ((r = dbus_message_new_method_return(m)) == NULL)
+			return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
+		dbus_message_iter_init_append(r, &iter);
+		dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY, "{oa{sa{sv}}}", &array);
+
+		for (i = 0; a2dp_codecs[i]; i++) {
+			const struct a2dp_codec *codec = a2dp_codecs[i];
+			uint8_t caps[A2DP_MAX_CAPS_SIZE];
+			int caps_size;
+			uint16_t codec_id = codec->id.codec_id;
+
+			caps_size = codec->fill_caps(0, caps);
+			if (caps_size < 0)
+				continue;
+
+			endpoint = spa_aprintf("%s/%s", A2DP_SINK_ENDPOINT, codec->name);
+			append_a2dp_object(&array, endpoint, SPA_BT_UUID_A2DP_SINK, codec_id, caps, caps_size);
+			free(endpoint);
+
+			endpoint = spa_aprintf("%s/%s", A2DP_SOURCE_ENDPOINT, codec->name);
+			append_a2dp_object(&array, endpoint, SPA_BT_UUID_A2DP_SOURCE, codec_id, caps, caps_size);
+			free(endpoint);
+		}
+
+		dbus_message_iter_close_container(&iter, &array);
+		if (!dbus_connection_send(monitor->conn, r, NULL))
+			return DBUS_HANDLER_RESULT_NEED_MEMORY;
+		res = DBUS_HANDLER_RESULT_HANDLED;
+	}
+	else
+		res = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+
+	return res;
+}
+
+static void bluez_register_application_reply(DBusPendingCall *pending, void *user_data)
+{
+	char *endpoint_path;
+	struct spa_bt_adapter *adapter = user_data;
+	struct spa_bt_monitor *monitor = adapter->monitor;
+	DBusMessage *r;
+	bool fallback = true;
+	int i;
+
+	r = dbus_pending_call_steal_reply(pending);
+	if (r == NULL)
+		return;
+
+	if (dbus_message_is_error(r, BLUEZ_ERROR_NOT_SUPPORTED)) {
+		spa_log_warn(monitor->log, "Registering media applications for adapter %s is disabled in bluez5", adapter->path);
+		goto finish;
+	}
+
+	if (dbus_message_get_type(r) == DBUS_MESSAGE_TYPE_ERROR) {
+		spa_log_error(monitor->log, "RegisterApplication() failed: %s",
+		        dbus_message_get_error_name(r));
+		goto finish;
+	}
+
+	fallback = false;
+	adapter->application_registered = true;
+	/* The application was registered in bluez5 - time to register the endpoints */
+	for (i = 0; a2dp_codecs[i]; i++) {
+		const struct a2dp_codec *codec = a2dp_codecs[i];
+
+		register_a2dp_endpoint(monitor, codec, A2DP_SOURCE_ENDPOINT, &endpoint_path);
+		if (endpoint_path)
+			free(endpoint_path);
+
+		register_a2dp_endpoint(monitor, codec, A2DP_SINK_ENDPOINT, &endpoint_path);
+		if (endpoint_path)
+			free(endpoint_path);
+	}
+
+	finish:
+	dbus_message_unref(r);
+	dbus_pending_call_unref(pending);
+
+	if (fallback)
+		adapter_register_endpoints(adapter);
+}
+
+static int adapter_register_application(struct spa_bt_adapter *a) {
+	const char *object_manager_path = A2DP_OBJECT_MANAGER_PATH;
+	struct spa_bt_monitor *monitor = a->monitor;
+	const DBusObjectPathVTable vtable_object_manager = {
+		.message_function = object_manager_handler,
+	};
+	DBusMessage *m;
+	DBusMessageIter i, d;
+	DBusPendingCall *call;
+
+	if (a->application_registered)
+		return 0;
+
+	spa_log_debug(monitor->log, "Registering bluez5 media application on adapter %s", a->path);
+
+	if (!dbus_connection_register_object_path(monitor->conn,
+	                                          object_manager_path,
+	                                          &vtable_object_manager, monitor))
+	return -EIO;
+
+	m = dbus_message_new_method_call(BLUEZ_SERVICE,
+	                                 a->path,
+	                                 BLUEZ_MEDIA_INTERFACE,
+	                                 "RegisterApplication");
+	if (m == NULL) {
+		dbus_connection_unregister_object_path(monitor->conn, object_manager_path);
+		return -EIO;
+	}
+
+	dbus_message_iter_init_append(m, &i);
+	dbus_message_iter_append_basic(&i, DBUS_TYPE_OBJECT_PATH, &object_manager_path);
+	dbus_message_iter_open_container(&i, DBUS_TYPE_ARRAY, "{sv}", &d);
+	dbus_message_iter_close_container(&i, &d);
+
+	dbus_connection_send_with_reply(monitor->conn, m, &call, -1);
+	dbus_pending_call_set_notify(call, bluez_register_application_reply, a, NULL);
+	dbus_message_unref(m);
+
+	return 0;
+}
 
 static void interface_added(struct spa_bt_monitor *monitor,
 			    DBusConnection *conn,
@@ -1416,7 +1459,7 @@ static void interface_added(struct spa_bt_monitor *monitor,
 			}
 		}
 		adapter_update_props(a, props_iter, NULL);
-		adapter_register_endpoints(a);
+		adapter_register_application(a);
 	}
 	else if (strcmp(interface_name, BLUEZ_PROFILE_MANAGER_INTERFACE) == 0) {
 		backend_hsp_native_register_profiles(monitor->backend_hsp_native);
@@ -1766,7 +1809,7 @@ impl_init(const struct spa_handle_factory *factory,
 
 	this->backend_hsp_native = backend_hsp_native_new(this, this->conn, support, n_support);
 	this->backend_ofono = backend_ofono_new(this, this->conn, support, n_support);
-	this->backend_hsphfpd = backend_hsphfpd_new(this, this->conn, support, n_support);
+	this->backend_hsphfpd = backend_hsphfpd_new(this, this->conn, info, support, n_support);
 
 	return 0;
 }
