@@ -74,6 +74,7 @@ static GstPipeWireCore *make_core (int fd)
   core->fd = fd;
   core->loop = pw_thread_loop_new ("pipewire-main-loop", NULL);
   core->context = pw_context_new (pw_thread_loop_get_loop(core->loop), NULL, 0);
+  core->last_seq = -1;
   GST_DEBUG ("loop %p context %p", core->loop, core->context);
 
   if (pw_thread_loop_start (core->loop) < 0)
@@ -101,12 +102,18 @@ static GstPipeWireCore *make_core (int fd)
 mainloop_failed:
   {
     GST_ERROR ("error starting mainloop");
+    pw_context_destroy (core->context);
+    pw_thread_loop_destroy (core->loop);
+    g_free (core);
     return NULL;
   }
 connection_failed:
   {
     GST_ERROR ("error connect: %m");
     pw_thread_loop_unlock (core->loop);
+    pw_context_destroy (core->context);
+    pw_thread_loop_destroy (core->loop);
+    g_free (core);
     return NULL;
   }
 }
