@@ -68,8 +68,9 @@ static void core_event_error(void *data, uint32_t id, int seq, int res, const ch
 
 	proxy = pw_map_lookup(&this->objects, id);
 
-	pw_log_error(NAME" %p: proxy %p id:%u: seq:%d res:%d (%s) msg:\"%s\"",
-			this, proxy, id, seq, res, spa_strerror(res), message);
+	pw_log_error(NAME" %p: proxy %p id:%u: bound:%d seq:%d res:%d (%s) msg:\"%s\"",
+			this, proxy, id, proxy ? proxy->bound_id : SPA_ID_INVALID,
+			seq, res, spa_strerror(res), message);
 	if (proxy)
 		pw_proxy_emit_error(proxy, seq, res, message);
 }
@@ -89,7 +90,7 @@ static void core_event_bound_id(void *data, uint32_t id, uint32_t global_id)
 	struct pw_core *this = data;
 	struct pw_proxy *proxy;
 
-	pw_log_debug(NAME" %p: proxy %u bound %u", this, id, global_id);
+	pw_log_debug(NAME" %p: proxy id %u bound %u", this, id, global_id);
 	if ((proxy = pw_map_lookup(&this->objects, id)) != NULL) {
 		pw_proxy_set_bound_id(proxy, global_id);
 	}
@@ -106,6 +107,7 @@ static void core_event_add_mem(void *data, uint32_t id, uint32_t type, int fd, u
 	if (m->id != id) {
 		pw_log_error(NAME" %p: invalid mem id %u, expected %u",
 				this, id, m->id);
+		pw_proxy_errorf(&this->proxy, -EINVAL, "invalid mem id %u, expected %u", id, m->id);
 		pw_memblock_unref(m);
 	}
 }
@@ -114,7 +116,7 @@ static void core_event_remove_mem(void *data, uint32_t id)
 {
 	struct pw_core *this = data;
 	pw_log_debug(NAME" %p: remove mem %u", this, id);
-	pw_mempool_unref_id(this->pool, id);
+	pw_mempool_remove_id(this->pool, id);
 }
 
 static const struct pw_core_events core_events = {
@@ -172,26 +174,39 @@ static int remove_proxy(void *object, void *data)
 	if (object == NULL)
 		return 0;
 
-	if (object != core) {
-		p->core = NULL;
+	if (object != core)
 		pw_proxy_remove(p);
-	}
 
 	return 0;
 }
 
-static void proxy_core_destroy(void *data)
+static int destroy_proxy(void *object, void *data)
+{
+	struct pw_core *core = data;
+	struct pw_proxy *p = object;
+
+	if (object == NULL)
+		return 0;
+
+	if (object != core) {
+		pw_log_warn(NAME" %p: leaked proxy %p id:%d", core, p, p->id);
+		p->core = NULL;
+	}
+	return 0;
+}
+
+static void proxy_core_removed(void *data)
 {
 	struct pw_core *core = data;
 	struct pw_stream *stream, *s2;
 	struct pw_filter *filter, *f2;
 
-	if (core->destroyed)
+	if (core->removed)
 		return;
 
-	core->destroyed = true;
+	core->removed = true;
 
-	pw_log_debug(NAME" %p: core proxy destroy", core);
+	pw_log_debug(NAME" %p: core proxy removed", core);
 	spa_list_remove(&core->link);
 
 	spa_list_for_each_safe(stream, s2, &core->stream_list, link)
@@ -200,16 +215,33 @@ static void proxy_core_destroy(void *data)
 		pw_filter_disconnect(filter);
 
 	pw_map_for_each(&core->objects, remove_proxy, core);
-	pw_map_reset(&core->objects);
+}
+
+static void proxy_core_destroy(void *data)
+{
+	struct pw_core *core = data;
+	struct pw_stream *stream;
+	struct pw_filter *filter;
+
+	if (core->destroyed)
+		return;
+
+	core->destroyed = true;
+
+	pw_log_debug(NAME" %p: core proxy destroy", core);
 
 	spa_list_consume(stream, &core->stream_list, link)
 		pw_stream_destroy(stream);
 	spa_list_consume(filter, &core->filter_list, link)
 		pw_filter_destroy(filter);
 
-	pw_protocol_client_disconnect(core->conn);
 	pw_proxy_destroy((struct pw_proxy*)core->client);
 
+	pw_map_for_each(&core->objects, destroy_proxy, core);
+	pw_map_reset(&core->objects);
+
+	pw_protocol_client_disconnect(core->conn);
+
 	pw_mempool_destroy(core->pool);
 
 	pw_protocol_client_destroy(core->conn);
@@ -218,10 +250,14 @@ static void proxy_core_destroy(void *data)
 
 	pw_log_debug(NAME" %p: free", core);
 	pw_properties_free(core->properties);
+
+	spa_hook_remove(&core->core_listener);
+	spa_hook_remove(&core->proxy_core_listener);
 }
 
 static const struct pw_proxy_events proxy_core_events = {
 	PW_VERSION_PROXY_EVENTS,
+	.removed = proxy_core_removed,
 	.destroy = proxy_core_destroy,
 };
 
@@ -257,6 +293,7 @@ struct pw_proxy *pw_core_export(struct pw_core *core,
 		res = -errno;
 		goto error_proxy_failed;
 	}
+	pw_log_debug(NAME" %p: export:%s proxy:%p", core, type, proxy);
 	return proxy;
 
 error_export_type:
@@ -344,7 +381,7 @@ error_properties:
 	pw_log_error(NAME" %p: can't create properties: %m", p);
 	goto exit_free;
 error_protocol:
-	pw_log_error(NAME" %p: can't find native protocol: %s", p, spa_strerror(res));
+	pw_log_error(NAME" %p: can't find protocol '%s': %s", p, protocol_name, spa_strerror(res));
 	goto exit_free;
 error_connection:
 	res = -errno;
@@ -375,6 +412,8 @@ pw_context_connect(struct pw_context *context, struct pw_properties *properties,
 	if (core == NULL)
 		return NULL;
 
+	pw_log_debug(NAME" %p: connect", core);
+
 	if ((res = pw_protocol_client_connect(core->conn,
 					&core->properties->dict,
 					NULL, NULL)) < 0)
@@ -400,6 +439,8 @@ pw_context_connect_fd(struct pw_context *context, int fd, struct pw_properties *
 	if (core == NULL)
 		return NULL;
 
+	pw_log_debug(NAME" %p: connect fd:%d", core, fd);
+
 	if ((res = pw_protocol_client_connect_fd(core->conn, fd, true)) < 0)
 		goto error_free;
 
@@ -429,12 +470,15 @@ pw_context_connect_self(struct pw_context *context, struct pw_properties *proper
 SPA_EXPORT
 int pw_core_steal_fd(struct pw_core *core)
 {
-	return pw_protocol_client_steal_fd(core->conn);
+	int fd = pw_protocol_client_steal_fd(core->conn);
+	pw_log_debug(NAME" %p: fd:%d", core, fd);
+	return fd;
 }
 
 SPA_EXPORT
 int pw_core_set_paused(struct pw_core *core, bool paused)
 {
+	pw_log_debug(NAME" %p: state:%s", core, paused ? "pause" : "resume");
 	return pw_protocol_client_set_paused(core->conn, paused);
 }
 
