@@ -190,6 +190,9 @@ void pw_impl_device_destroy(struct pw_impl_device *device)
 	if (device->registered)
 		spa_list_remove(&device->link);
 
+	if (device->device)
+		spa_hook_remove(&device->listener);
+
 	if (device->global) {
 		spa_hook_remove(&device->global_listener);
 		pw_global_destroy(device->global);
@@ -197,6 +200,8 @@ void pw_impl_device_destroy(struct pw_impl_device *device)
 	pw_log_debug(NAME" %p: free", device);
 	pw_impl_device_emit_free(device);
 
+	spa_hook_list_clean(&device->listener_list);
+
 	pw_properties_free(device->properties);
 	free(device->name);
 
@@ -216,6 +221,8 @@ static void resource_destroy(void *data)
 {
 	struct resource_data *d = data;
 	remove_busy_resource(d);
+	spa_hook_remove(&d->resource_listener);
+	spa_hook_remove(&d->object_listener);
 }
 
 static void resource_pong(void *data, int seq)
@@ -479,13 +486,6 @@ int pw_impl_device_register(struct pw_impl_device *device,
 	if (device->registered)
 		goto error_existed;
 
-	if (properties == NULL)
-		properties = pw_properties_new(NULL, NULL);
-	if (properties == NULL)
-		return -errno;
-
-	pw_properties_update_keys(properties, &device->properties->dict, keys);
-
         device->global = pw_global_new(context,
 				       PW_TYPE_INTERFACE_Device,
 				       PW_VERSION_DEVICE,
@@ -502,6 +502,8 @@ int pw_impl_device_register(struct pw_impl_device *device,
 	pw_properties_setf(device->properties, PW_KEY_OBJECT_ID, "%d", device->info.id);
 	device->info.props = &device->properties->dict;
 
+	pw_global_update_keys(device->global, device->info.props, keys);
+
 	pw_impl_device_emit_initialized(device);
 
 	pw_global_add_listener(device->global, &device->global_listener, &global_events, device);
