@@ -635,7 +635,8 @@ static void on_core_error(void *userdata, uint32_t id, int seq, int res, const c
 	fprintf(stderr, "remote error: id=%"PRIu32" seq:%d res:%d (%s): %s\n",
 			id, seq, res, spa_strerror(res), message);
 
-	pw_main_loop_quit(data->loop);
+	if (id == PW_ID_CORE && res == -EPIPE)
+		pw_main_loop_quit(data->loop);
 }
 
 static const struct pw_core_events core_events = {
@@ -1103,6 +1104,53 @@ static int setup_midifile(struct data *data)
 	return 0;
 }
 
+static int fill_properties(struct data *data)
+{
+	static const char* table[] = {
+		[SF_STR_TITLE] = PW_KEY_MEDIA_TITLE,
+		[SF_STR_COPYRIGHT] = PW_KEY_MEDIA_COPYRIGHT,
+		[SF_STR_SOFTWARE] = PW_KEY_MEDIA_SOFTWARE,
+		[SF_STR_ARTIST] = PW_KEY_MEDIA_ARTIST,
+		[SF_STR_COMMENT] = PW_KEY_MEDIA_COMMENT,
+		[SF_STR_DATE] = PW_KEY_MEDIA_DATE
+	};
+	SF_INFO sfi;
+	SF_FORMAT_INFO fi;
+	int res;
+	unsigned c;
+	const char *s, *t;
+
+	for (c = 0; c < SPA_N_ELEMENTS(table); c++) {
+		if (table[c] == NULL)
+			continue;
+
+		if ((s = sf_get_string(data->file, c)) == NULL ||
+		    *s == '\0')
+			continue;
+
+		pw_properties_set(data->props, table[c], s);
+	}
+
+	spa_zero(sfi);
+	if ((res = sf_command(data->file, SFC_GET_CURRENT_SF_INFO, &sfi, sizeof(sfi)))) {
+		pw_log_error("sndfile: %s", sf_error_number(res));
+		return -EIO;
+	}
+
+	spa_zero(fi);
+	fi.format = sfi.format;
+	if (sf_command(data->file, SFC_GET_FORMAT_INFO, &fi, sizeof(fi)) == 0 && fi.name)
+		pw_properties_set(data->props, PW_KEY_MEDIA_FORMAT, fi.name);
+
+	s = pw_properties_get(data->props, PW_KEY_MEDIA_TITLE);
+	t = pw_properties_get(data->props, PW_KEY_MEDIA_ARTIST);
+	if (s && t)
+		pw_properties_setf(data->props, PW_KEY_MEDIA_NAME,
+				"'%s' / '%s'", s, t);
+
+	return 0;
+}
+
 static int setup_sndfile(struct data *data)
 {
 	SF_INFO info;
@@ -1179,6 +1227,7 @@ static int setup_sndfile(struct data *data)
 				printf("\n");
 			}
 		}
+		fill_properties(data);
 	}
 	data->samplesize = sf_format_samplesize(info.format);
 	data->stride = data->samplesize * data->channels;
@@ -1460,6 +1509,7 @@ int main(int argc, char *argv[])
 			PW_KEY_MEDIA_CATEGORY, data.media_category,
 			PW_KEY_MEDIA_ROLE, data.media_role,
 			PW_KEY_APP_NAME, prog,
+			PW_KEY_MEDIA_FILENAME, data.filename,
 			PW_KEY_MEDIA_NAME, data.filename,
 			PW_KEY_NODE_NAME, prog,
 			NULL);
